/**
 * IntTextField.java
 */
package UTEStatsViewer;
import java.awt.*;
import java.awt.event.*;

/**
 * A text field that may contain an integer value within a given range.
 */
public class IntTextField extends TextField implements ActionListener,
      FocusListener
{
   // the minimum and maximum allowable values
   private int min, max;

   // the most recent correct value of the field
   private int value;

   /** Return the most recent correct value. */
   public final int getValue() {return value;}

   // whether the value in the field is valid
   private boolean valueIsValid = true;

   /** Return whether value is valid. */
   public final boolean isValueValid() {return valueIsValid;}

   // background color used when value is valid
   private Color okBackground;


   /**
    * Create a text field that will accept integer values within a
    * given range.
    * @param min the minimum allowable value
    * @param max the maximum allowable value
    * @param value the initial value of the TextField
    * @param cols the width of the TextField in terms of colums
    */
   public IntTextField(int min, int max, String value, int cols) {
      super(value, cols);
      this.min = min;
      this.max = max;
      addActionListener(this);
      addFocusListener(this);
      okBackground = getBackground();
      validateValue();
   }


   /**
    * Create a text field that will accept integer values within a
    * given range.
    * @param min the minimum allowable value
    * @param max the maximum allowable value
    * @param value the initial value of the TextField
    */
   public IntTextField(int min, int max, String value) {
      this(min, max, String.valueOf(value), 
            Math.max(value.length(), Math.max(
                  String.valueOf(min).length(),
                  String.valueOf(max).length()))
           );
   }


   /** Override setText(String) to validate the value being set. */
   public void setText(String text) {
      super.setText(text);
      validateValue();
   }


   /** Validates whether current value in field is valid. */
   private boolean validateValue() {
      int value = 0;
      boolean valueIsValid;

      // get the value from the field
      try {
         value = Integer.parseInt(getText());
         valueIsValid = ((value >= min) && (value <= max));
      } catch (NumberFormatException e) {
         valueIsValid = false;
      }
      if (valueIsValid) {
         this.value = value; 
      }

      // if validity of the field value has changed
      if (valueIsValid != this.valueIsValid) {
         if (valueIsValid) {
            // value used to be invalid, restore foreground
            setBackground(okBackground);
         } else {
            // value invalid, use complemented foreground
            setBackground(new Color(0x00ffffff ^ getForeground().getRGB()));
            getToolkit().beep();
         }
         this.valueIsValid = valueIsValid;
      }

      return valueIsValid;
   }


   /** Implement the ActionListener interface. */
   public void actionPerformed(ActionEvent e) {
      validateValue(); 
   }


   /** Implement the FocusListener interface. */
   public void focusGained(FocusEvent e) {}

   public void focusLost(FocusEvent e) {
      validateValue(); 
   }
}
